@@ -1,7 +1,7 @@
 /*
  * augeas.h: public headers for augeas
  *
- * Copyright (C) 2007-2015 David Lutterkort
+ * Copyright (C) 2007-2016 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -396,6 +396,19 @@ int aug_escape_name(augeas *aug, const char *in, char **out);
  */
 int aug_print(const augeas *aug, FILE *out, const char *path);
 
+/* Function: aug_source
+ *
+ * For the node matching PATH, return the path to the node representing the
+ * file to which PATH belongs. If PATH belongs to a file, *FILE_PATH will
+ * contain the path to the toplevel node of that file underneath /files. If
+ * it does not, *FILE_PATH will be NULL.
+ *
+ * Returns:
+ * 0 on success, or a negative value on failure. It is an error if PATH
+ * matches more than one node.
+ */
+int aug_source(const augeas *aug, const char *path, char **file_path);
+
 /* Function: aug_to_xml
  *
  * Turn the Augeas tree(s) matching PATH into an XML tree XMLDOC. The
@@ -423,6 +436,21 @@ int aug_to_xml(const augeas *aug, const char *path, xmlNode **xmldoc,
  */
 int aug_transform(augeas *aug, const char *lens, const char *file, int excl);
 
+/*
+ * Function: aug_load_file
+ *
+ * Load a FILE using the lens that would ordinarily be used by aug_load,
+ * i.e. the lens whose autoload statement matches the FILE. Similar to
+ * aug_load, this function returns successfully even if FILE does not exist
+ * or if the FILE can not be processed by the associated lens. It is an
+ * error though if no lens can be found to process FILE. In that case, the
+ * error code in AUG will be set to AUG_ENOLENS.
+ *
+ * Returns:
+ * 0 on success, -1 on failure
+ */
+int aug_load_file(augeas *aug, const char *file);
+
 /*
  * Function: aug_srun
  *
@@ -445,6 +473,99 @@ int aug_srun(augeas *aug, FILE *out, const char *text);
  */
 void aug_close(augeas *aug);
 
+// We can't put //* into the examples in these comments since the C
+// preprocessor complains about that. So we'll resort to the equivalent but
+// more wordy notation /descendant::*
+
+/*
+ * Function: aug_ns_attr
+ *
+ * Look up the ith node in the variable VAR and retrieve information about
+ * it. Set *VALUE to the value of the node, *LABEL to its label, and
+ * *FILE_PATH to the path of the file it belongs to, or to NULL if that
+ * node does not belong to a file. It is permissible to pass NULL for any
+ * of these variables to indicate that the caller is not interested in that
+ * attribute.
+ *
+ * It is assumed that VAR was defined with a path expression evaluating to
+ * a nodeset, like '/files/etc/hosts/descendant::*'. This function is
+ * equivalent to, but faster than, aug_get(aug, "$VAR[I+1]", value),
+ * respectively the corresponding calls to aug_label and aug_source. Note
+ * that the index is 0-based, not 1-based.
+ *
+ * If VAR does not exist, or is not a nodeset, or if it has fewer than I
+ * nodes, this call fails.
+ *
+ * The caller is responsible for freeing *FILE_PATH, but must not free
+ * *VALUE or *LABEL. Those pointers are only valid up to the next call to a
+ * function in this API that might modify the tree.
+ *
+ * Returns:
+ * 1 on success (for consistency with aug_get), a negative value on failure
+ */
+int aug_ns_attr(const augeas* aug, const char *var, int i,
+                const char **value, const char **label, char **file_path);
+
+/*
+ * Function: aug_ns_label
+ *
+ * Look up the LABEL and its INDEX amongst its siblings for the ith node in
+ * variable VAR. (See aug_ns_attr for details of what is expected of VAR)
+ *
+ * Either of LABEL and INDEX may be NULL. The *INDEX will be set to the
+ * number of siblings + 1 of the node $VAR[I+1] that precede it and have
+ * the same label if there are at least two siblings with that label. If
+ * the node $VAR[I+1] does not have any siblings with the same label as
+ * itself, *INDEX will be set to 0.
+ *
+ * The caller must not free *LABEL. The pointer is only valid up to the
+ * next call to a function in this API that might modify the tree.
+ *
+ * Returns:
+ * 1 on success (for consistency with aug_get), a negative value on failure
+ */
+int aug_ns_label(const augeas *aug, const char *var, int i,
+                 const char **label, int *index);
+
+/*
+ * Function: aug_ns_value
+ *
+ * Look up the VALUE of the ith node in variable VAR. (See aug_ns_attr for
+ * details of what is expected of VAR)
+ *
+ * The caller must not free *VALUE. The pointer is only valid up to the
+ * next call to a function in this API that might modify the tree.
+ *
+ * Returns:
+ * 1 on success (for consistency with aug_get), a negative value on failure
+ */
+int aug_ns_value(const augeas *aug, const char *var, int i,
+                 const char **value);
+
+/*
+ * Function: aug_ns_count
+ *
+ * Return the number of nodes in variable VAR. (See aug_ns_attr for details
+ * of what is expected of VAR)
+ *
+ * Returns: the number of nodes in VAR, or a negative value on failure
+ */
+int aug_ns_count(const augeas *aug, const char *var);
+
+/*
+ * Function: aug_ns_count
+ *
+ * Put the fully qualified path to the ith node in VAR into *PATH. (See
+ * aug_ns_attr for details of what is expected of VAR)
+ *
+ * The caller is responsible for freeing *PATH, which is allocated by this
+ * function.
+ *
+ * Returns: 1 on success (for consistency with aug_get), a negative value
+ * on failure
+ */
+int aug_ns_path(const augeas *aug, const char *var, int i, char **path);
+
 /*
  * Error reporting
  */
@@ -462,7 +583,7 @@ typedef enum {
     AUG_ENOSPAN,        /* No span for this node */
     AUG_EMVDESC,        /* Cannot move node into its descendant */
     AUG_ECMDRUN,        /* Failed to execute command */
-    AUG_EBADARG,        /* Invalid argument in funcion call */
+    AUG_EBADARG,        /* Invalid argument in function call */
     AUG_ELABEL,         /* Invalid label */
     AUG_ECPDESC         /* Cannot copy node into its descendant */
 } aug_errcode_t;
